#!/usr/bin/perl
use Getopt::Long;
use File::Path;
use Config;

sub help;
sub expvar($);
sub makedir($);
sub copyfiles($$$$);

$result = GetOptions 'help', 'texlive|tl|tetex', 'miktex', 'tdscompliant',
         'source=s', 'target=s', 'withxetex|withxelatex', 'noxetex|noxelatex',
         'czech', 'slovak', 'cs', 'hindi', 'remove',
         'xelatex', 'latex', 'pdflatex';

help if $opt_help || !$result;

# Analyze system options
$a = 0 + $opt_texlive + $opt_miktex + $opt_tdscompliant;
die  "System options are mutually exclusive!\n" if $a > 1;
$opt_texlive = 1 unless $a;

# Analyze engine options
$a = 0 + $opt_withxetex + $opt_noxetex;
die "Engine options are mutually exclusive!\n" if $a > 1;
$opt_withxetex = $opt_texlive unless $a;
$opt_noxetex= $opt_miktex || $opt_tdscompliant unless $a;

# Analyze language options
$opt_czech = $opt_slovak = 1 if $opt_cs;
die "--remove is mutually exclusive with other language options!\n"
    if $opt_remove && ($opt_czech || $opt_slovak || $opt_hindi);
$lang = $opt_czech || $opt_slovak || $opt_hindi || $opt_remove;

# Analyze format options
$fmt = $opt_latex || $opt_pdflatex || $opt_xelatex;
die "Formats can be generated for --texlive only!\n" if $fmt && !$opt_texlive;

# Analyze directory options
die "No target specified!\n" if $lang && !$opt_target && !$opt_texlive;
die "Source not specified!\n" if $lang && $opt_tdscompliant && !$opt_source;

# Useful constants
$tgdir = 'csh-babel';	# target directory
$hpdir = 'csh-hyphen';	# hyph directory
$vardir = 'TEXMFVAR';
$locdir = 'TEXMFLOCAL';
$hmdir = 'TEXMFHOME';
$lgdat = 'language.dat';
$generic = 'tex/generic';
$cfg = 'config';
$wcdir = 'web2c';

# Process languages

$osname = lc $Config{'osname'};
$cp = ($osname =~ /(os\/2|windows|cygwin|mswin)/) ? 'copy' : 'cp -p';
if ($lang) {

  # Removing the files, all locations will be tried
  if ($opt_remove) {
    my @tdirs, @lgdirs;
    if ($opt_texlive) {
      my $a = expvar $hmdir;
      push @tdirs, $a;
      push @lgdirs, $a;
      $a = expvar $vardir;
      push @tgdirs, $a;
      push @lgdirs, $a;
      $a = expvar $locdir;
      push @tdirs, $a;
    } else {
      push @tdirs, $opt_target;
      push @lgdirs, $opt_target;
    }

    # Removing language.dat, will be renamed to csh-language.dat
    foreach (@lgdirs) {
      my $tg = "$_/$generic/$cfg";
      if (-f "$tg/$lgdat") {
        unlink "$tg/csh-$lgdat" if -f "$tg/csh-$lgdat";
        rename "$tg/$lgdat", "$tg/csh-$lgdat" unless -f "$tg/csh-$lgdat";
        warn "Could not remove $tg/$lgdat\n" if -f "$tg/$lgdat";
      }
    }

    # Removing the directories
    foreach (@tdirs) {
      my $tg = "$_/$generic/$tgdir";
      if (-d $tg) {
        rmtree $tg, 0, 0;
        warn "Could not remove $tg\n" if -d $tg;
      }
      $tg = "$_/$generic/$hpdir";
      if (-d $tg) {
        rmtree $tg, 0, 0;
        warn "Could not remove $tg\n" if -d $tg;
      }
    }
  } else {

    # Find target directories
    my $lgdir, $txdir;
    if ($opt_texlive) {
      $txdir = $lgdir = expvar $vardir;
      if (-d $txdir && -w $txdir) {
        print "$txdir is writable.\n";
      } else {
        $txdir = '';
      }
      if (!$txdir) {
        $txdir = $lgdir = expvar $hmdir;
        print "Files will be installed in $tgdir.\n";
      }
    } else {
      $lgdir = $txdir = $opt_target;
    }

    # Making the directories
    my $tg = makedir "$txdir/$generic/$tgdir";
    my $hp = makedir "$txdir/$generic/$hpdir";
    my $lg = makedir "$lgdir/$generic/$cfg";
    my $wc = makedir "$lgdir/$wcdir" if $opt_czech || $opt_slovak;

    # Copying the files
    copyfiles $tg, $hp, 'czech', 'cz' if $opt_czech;
    copyfiles $tg, $hp, 'slovak', 'sk' if $opt_slovak;
    copyfiles $tg, $hp, 'hindi', '' if $opt_hindi;
    if ($opt_czech || $opt_slovak) {
      foreach (qw{babel.def switch.def hyphen.cfg}) {
        system "$cp $_ $tg";
      }
      system "$cp *.tcx $wc";
    }

    # Processing the language.dat file
    my $olg;
    chomp($olg = `kpsewhich $lgdat`) if $opt_texlive;
    chomp($olg = `findtexmf $lgdat`) if $opt_miktex;
    $olg = "$opt_source/$generic/$cfg/$lgdat" if $opt_tdscompliant;
    die "Cannot locate $lgdat\n" unless $olg;
    die "File $olg does not exist.\n" unless -f $olg;

    # We might be overwriting the same file
    my @lgdat;
    open(LG, $olg) or die "Cannot read from $olg\n";
    while (<LG>) {
      push @lgdat, $_;
    }
    close LG;
    open(LG, ">$lg/$lgdat") or die "Cannot write to $lg/$lgdat\n";
    foreach (@lgdat) {
      print LG "%-- " if $opt_czech && /czech/;
      print LG "%-- " if $opt_slovak && /slovak/;
      print LG "%-- " if $opt_hindi && /hindi/;
      print LG;
    }
    print LG "%++ Czech/Slovak/Hindi additions\n";
    if ($opt_czech) {
      print LG "% Czech from csbabel 3.1, see www.cstug.cz\n";
      print LG "czech:IL2 czhyph.tex\n";
      print LG "czech:T1 czhyph.tex\n";
      print LG "czechX czhyph.tex\n" if $opt_withxetex;
      print LG "=czech\n";
    }
    if ($opt_slovak) {
      print LG "% Slovak from csbabel 3.1, see www.cstug.cz\n";
      print LG "slovak:IL2 skhyph.tex\n";
      print LG "slovak:T1 skhyph.tex\n";
      print LG "slovakX skhyph.tex\n" if $opt_withxetex;
      print LG "=slovak\n";
    }
    if ($opt_hindi) {
      print LG "% Hindi, see http://devnag.sarovar.org and\n";
      print LG "%            http://icebearsoft.euweb.cz\n";
      print LG "hindi zerohyph.tex\n";
    }
    print LG "%++ End of Czech/Slovak/Hindi additions\n";
    close LG;
  }

  # Filename database
  if ($opt_texlive) {
    system 'mktexlsr';
  } else {
    print "Refresh your filename database.\n";
  }
}

# Making the formats for TeXLive
if ($opt_texlive) {
  system "fmtutil --byfmt latex" if $opt_latex;
  system "fmtutil --byfmt pdflatex" if $opt_pdflatex;
  system "fmtutil --byfmt xelatex" if $opt_xelatex;
}
exit 0;

sub help {
print<<EOD;
Usage (see alse http://icebearsoft.euweb.cz/tex/csh_babel.php):

System options (mutually exclusive):
  --texlive	TeXLive 2007 (default)
  --tl		alias for --texlive
  --tetex	alias for --texlive
  --miktex	MiKTeX (files will be searched by findtexmf)
  --tdscompliant other TDS compliant distribution

Directories options:
  --source	source directory root for TDS compliant distribution
		ignored otherwise
  --target	target directory tree, ignored for TeXLive

Engine options (mutually exclusive):
  --withxetex	make language.dat for XeTeX/XeLaTeX, --withxelatex is alias
		default for --texlive
  --noxetex	do not make language.dat for XeTeX/XeLaTeX, --noxelatex is
		alias, default for --miktex and --tdscompliant
  These options have no effect for Hindi

Language options:
  --czech	Czech
  --slovak	Slovak
  --cs		both --czech and --slovak
  --hindi	Hindi
  --remove	remove all installed files

Format options (for TeXLive only):
  --latex	generate latex.fmt
  --pdflatex	generate pdflatex.fmt
  --xelatex	generate xelatex.fmt

Note: If your distribution contains programs kpsewhich, mktexlsr, fmtutil, and
texmf.cnf defines variables TEXMFLOCAL and TEXMFHOME, you can use --texlive.
If only fmtutil is missing, you can still use --texlive but without format
options. The formats must be generated by other means.

Full documentation available from http://icebearsoft.euweb.cz/tex/csh_babel.php
EOD
exit;
}

sub expvar($) {
  my $var = shift;
  my $val;
  chomp($val = `kpsewhich --var-value $var`);
  die "Variable $var not found.\n" unless $val;
  return $val;
}

sub makedir($) {
  my $dir = shift;
  mkpath $dir unless -d $dir;
  die "Cannot create directory $dir, check the permissions.\n" unless -d $dir;
  $dir =~ s/\//\\/g if $cp eq 'copy';
  return $dir;
}

sub copyfiles($$$$) {
  my ($ldir, $hdir, $language, $prefix) = @_;
  system "$cp ${prefix}hyph.tex $hdir" if $prefix;
  foreach (qw{ldf sty}) {
    system "$cp $language.$_ $ldir";
  }
}
